Function alphanumeric0

fn alphanumeric0<Input, Error>(input: &mut Input) -> Result<<Input as Stream>::Slice, Error>
where
    Input: StreamIsPartial + Stream,
    <Input as Stream>::Token: AsChar,
    Error: ParserError<Input>

Recognizes zero or more ASCII numerical and alphabetic characters: 'a'..='z', 'A'..='Z', '0'..='9'

Complete version: Will return the whole input if no terminating token is found (a non alphanumerical character).

[Partial version][crate::_topic::partial]: Will return Err(winnow::error::ErrMode::Incomplete(_)) if there's not enough input data, or if no terminating token is found (a non alphanumerical character).

Effective Signature

Assuming you are parsing a &str [Stream]:

# use winnow::prelude::*;;
pub fn alphanumeric0<'i>(input: &mut &'i str) -> ModalResult<&'i str>
# {
#     winnow::ascii::alphanumeric0.parse_next(input)
# }

Example

# use winnow::prelude::*;
# use winnow::ascii::alphanumeric0;
fn parser<'s>(input: &mut &'s str) -> ModalResult<&'s str> {
    alphanumeric0.parse_next(input)
}

assert_eq!(parser.parse_peek("21cZ%1"), Ok(("%1", "21cZ")));
assert_eq!(parser.parse_peek("&Z21c"), Ok(("&Z21c", "")));
assert_eq!(parser.parse_peek(""), Ok(("", "")));
# use winnow::prelude::*;
# use winnow::{error::ErrMode, error::ContextError, error::Needed};
# use winnow::Partial;
# use winnow::ascii::alphanumeric0;
assert_eq!(alphanumeric0::<_, ErrMode<ContextError>>.parse_peek(Partial::new("21cZ%1")), Ok((Partial::new("%1"), "21cZ")));
assert_eq!(alphanumeric0::<_, ErrMode<ContextError>>.parse_peek(Partial::new("&Z21c")), Ok((Partial::new("&Z21c"), "")));
assert_eq!(alphanumeric0::<_, ErrMode<ContextError>>.parse_peek(Partial::new("")), Err(ErrMode::Incomplete(Needed::new(1))));