Function bytes
fn bytes<Input, Output, ByteError, BitError, ParseNext>(parser: ParseNext) -> impl Parser<(Input, usize), Output, BitError>
where
ByteError: ParserError<Input> + ErrorConvert<BitError>,
BitError: ParserError<(Input, usize)>,
Input: Stream<Token = u8> + Clone,
ParseNext: Parser<Input, Output, ByteError>
Convert a bits stream back into a byte stream
Warning: A partial byte remaining in the input will be ignored and the given parser will start parsing at the next full byte.
Examples
# use *;
# use Bytes;
# use rest;
# use ContextError;
# use ErrMode;
use ;
type Stream<'i> = &'i Bytes;
let input = stream;
assert_eq!;