Type Alias ModalResult
type ModalResult = Result<O, ErrMode<E>>
[Modal error reporting][ErrMode] for Parser::parse_next
Ok(O)is the parsed value- [
Err(ErrMode<E>)][ErrMode] is the error along with how to respond to it
By default, the error type (E) is ContextError.
When integrating into the result of the application, see