Type Alias Result
type Result = Result<O, E>
By default, the error type (E) is ContextError.
When integrating into the result of the application, see
Resulttype Result = Result<O, E>
By default, the error type (E) is ContextError.
When integrating into the result of the application, see