Function space1

fn space1<Input, Error>(input: &mut Input) -> Result<<Input as Stream>::Slice, Error>
where
    Input: StreamIsPartial + Stream,
    <Input as Stream>::Token: AsChar,
    Error: ParserError<Input>

Recognizes one or more spaces and tabs.

Complete version: Will return the whole input if no terminating token is found (a non space character).

[Partial version][crate::_topic::partial]: Will return Err(winnow::error::ErrMode::Incomplete(_)) if there's not enough input data, or if no terminating token is found (a non space character).

Effective Signature

Assuming you are parsing a &str [Stream]:

# use winnow::prelude::*;;
pub fn space1<'i>(input: &mut &'i str) -> ModalResult<&'i str>
# {
#     winnow::ascii::space1.parse_next(input)
# }

Example

# use winnow::prelude::*;
# use winnow::ascii::space1;
fn parser<'s>(input: &mut &'s str) -> ModalResult<&'s str> {
    space1.parse_next(input)
}

assert_eq!(parser.parse_peek(" \t21c"), Ok(("21c", " \t")));
assert!(parser.parse_peek("H2").is_err());
assert!(parser.parse_peek("").is_err());
# use winnow::prelude::*;
# use winnow::{error::ErrMode, error::ContextError, error::Needed};
# use winnow::Partial;
# use winnow::ascii::space1;
assert_eq!(space1::<_, ErrMode<ContextError>>.parse_peek(Partial::new(" \t21c")), Ok((Partial::new("21c"), " \t")));
assert!(space1::<_, ErrMode<ContextError>>.parse_peek(Partial::new("H2")).is_err());
assert_eq!(space1::<_, ErrMode<ContextError>>.parse_peek(Partial::new("")), Err(ErrMode::Incomplete(Needed::new(1))));