Trait AddContext

trait AddContext<I: Stream, C = &'static str>: Sized

Used by Parser::context to add custom data to error while backtracking

May be implemented multiple times for different kinds of context.

Provided Methods

fn add_context(self: Self, _input: &I, _token_start: &<I as Stream>::Checkpoint, _context: C) -> Self

Append to an existing error custom data

This is used mainly by Parser::context, to add user friendly information to errors when backtracking through a parse tree

Implementors