pub trait Offset<Start = Self> {
// Required method
fn offset_from(&self, start: &Start) -> usize;
}Expand description
Useful functions to calculate the offset between slices and show a hexdump of a slice
Required Methods
Sourcefn offset_from(&self, start: &Start) -> usize
fn offset_from(&self, start: &Start) -> usize
Offset between the first byte of start and the first byte of selfa
Note: This is an offset, not an index, and may point to the end of input
(start.len()) when self is exhausted.
Implementations on Foreign Types
Sourceimpl<'a> Offset<<&'a str as Stream>::Checkpoint> for &'a str
impl<'a> Offset<<&'a str as Stream>::Checkpoint> for &'a str
fn offset_from(&self, other: &<&'a str as Stream>::Checkpoint) -> usize
Sourceimpl<'a, T> Offset<<&'a [T] as Stream>::Checkpoint> for &'a [T]
impl<'a, T> Offset<<&'a [T] as Stream>::Checkpoint> for &'a [T]
fn offset_from(&self, other: &<&'a [T] as Stream>::Checkpoint) -> usize
Implementors
impl Offset for &BStr
impl Offset for &Bytes
impl<'a> Offset<<&'a BStr as Stream>::Checkpoint> for &'a BStr
impl<'a> Offset<<&'a Bytes as Stream>::Checkpoint> for &'a Bytes
impl<I> Offset for Bits<I>where
I: Offset,
Available on crate feature
binary only.impl<I> Offset for LocatingSlice<I>where
I: Stream,
impl<I> Offset for Partial<I>where
I: Stream,
impl<I> Offset<<Bits<I> as Stream>::Checkpoint> for Bits<I>
Available on crate feature
binary only.impl<I> Offset<<LocatingSlice<I> as Stream>::Checkpoint> for LocatingSlice<I>where
I: Stream,
impl<I> Offset<<Partial<I> as Stream>::Checkpoint> for Partial<I>where
I: Stream,
impl<I, E> Offset for Recoverable<I, E>
Available on crate features
unstable-recover and std only.impl<I, E> Offset<<Recoverable<I, E> as Stream>::Checkpoint> for Recoverable<I, E>
Available on crate features
unstable-recover and std only.