Trait Offset

trait Offset<Start = Self>

Useful functions to calculate the offset between slices and show a hexdump of a slice

Required Methods

fn offset_from(self: &Self, start: &Start) -> usize

Offset between the first byte of start and the first byte of selfa

Note: This is an offset, not an index, and may point to the end of input (start.len()) when self is exhausted.

Implementors