Trait Offset
trait Offset<Start = Self>
Useful functions to calculate the offset between slices and show a hexdump of a slice
Required Methods
fn offset_from(self: &Self, start: &Start) -> usizeOffset between the first byte of
startand the first byte ofselfaNote: This is an offset, not an index, and may point to the end of input (
start.len()) whenselfis exhausted.
Implementors
impl<T> Offset for TokenSlice<'_, T>impl<I, S> Offset for Stateful<I, S>impl<I> Offset for (I, usize)impl<T> Offset for &[T]impl<I> Offset for LocatingSlice<I>impl Offset for &Bytesimpl<I, S> Offset for Stateful<I, S>impl<'a> Offset for &'a strimpl<I> Offset for LocatingSlice<I>impl Offset for &BStrimpl<I, S> Offset for Checkpoint<I, S>impl<I> Offset for Partial<I>impl Offset for &strimpl<'a> Offset for &'a BStrimpl<T> Offset for TokenSlice<'_, T>impl<I> Offset for (I, usize)impl<I> Offset for Partial<I>impl<'a, T> Offset for &'a [T]impl<'a> Offset for &'a Bytes